//+------------------------------------------------------------------+
//|                                               ChannelPattern.mq4 |
//|           Copyright: Andriy Moraru, www.earnforex.com, 2013-2020 |
//|                                        https://www.earnforex.com |
//+------------------------------------------------------------------+
#property copyright "Copyright: Andriy Moraru, www.earnforex.com, 2013-2020"
#property link      "https://www.earnforex.com/blog/channelpattern-script-update-one-sided-breakouts/"
#property version   "1.07"
#property show_inputs
#property strict

#property description "Finds a channel - Border."
#property description "Creates two other channels:"
#property description " Entry at 10% of channel height (calculated using Border)."
#property description " TP at 100% of channel height (calculated using Border)."
#property description " If Sides != Both, trendlines will be used instead of channels."
#property description "If doesn't find Border channel, tries to find UpperBorder or LowerBorder trendlines or horizontal lines."
#property description "Automatically applies styling."

enum sides_enum
{
   Upper,
   Lower,
   Both
};

input string BorderName = "Border";
input string EntryName = "Entry";
input string TPName = "TP";
input int PercentageEntry = 10; // Distance to entry in % of the channel's height
input int PercentageTP = 100; // Distance to entry in % of the channel's height
input sides_enum Sides = Both;
input color EntryColor = clrTurquoise;
input color TPColor = clrLimeGreen;
input ENUM_LINE_STYLE EntryStyle = STYLE_DOT;
input ENUM_LINE_STYLE TPStyle = STYLE_DOT;
input string UpperBorder = "UpperBorder";
input string LowerBorder = "LowerBorder";
input string UpperEntry = "UpperEntry";
input string UpperTP = "UpperTP";
input string LowerEntry = "LowerEntry";
input string LowerTP = "LowerTP";
   
void OnStart()
{
   datetime T1, T2, T3;
   double P1, P2, P3, Height;

   Print("Channel Pattern script started.");

   if ((PercentageEntry < 0) || (PercentageTP < 0))
   {
      Print("Percentage values cannot be negative. Terminating script.");
      return;
   }

   if (ObjectFind(0, BorderName) >= 0)
   {
      Print("Border channel found.");
      T1 = (datetime)ObjectGetInteger(0, BorderName, OBJPROP_TIME, 0);
      T2 = (datetime)ObjectGetInteger(0, BorderName, OBJPROP_TIME, 1);
      T3 = (datetime)ObjectGetInteger(0, BorderName, OBJPROP_TIME, 2);
      P1 = ObjectGetDouble(0, BorderName, OBJPROP_PRICE, 0);
      P2 = ObjectGetDouble(0, BorderName, OBJPROP_PRICE, 1);
      P3 = ObjectGetDouble(0, BorderName, OBJPROP_PRICE, 2);
      // Get object's height, making sure that the channel is "rayed" and switching ray properties back as they were.
      bool prev_ray_right = ObjectGetInteger(0, BorderName, OBJPROP_RAY_RIGHT);
      ObjectSetInteger(0, BorderName, OBJPROP_RAY_RIGHT, true);
      Height = ObjectGetValueByTime(0, BorderName, T1, 0) - ObjectGetValueByTime(0, BorderName, T1, 1);
      ObjectSetInteger(0, BorderName, OBJPROP_RAY_RIGHT, prev_ray_right);
   }
   else if ((ObjectFind(0, UpperBorder) >= 0) && (ObjectFind(0, LowerBorder) >= 0))
   {
      Print("Border lines found.");
      T1 = (datetime)ObjectGetInteger(0, UpperBorder, OBJPROP_TIME, 0);
      T2 = (datetime)ObjectGetInteger(0, UpperBorder, OBJPROP_TIME, 1);
      T3 = (datetime)ObjectGetInteger(0, LowerBorder, OBJPROP_TIME, 0);
      // Horizontal line case
      if (((int)T2 == 0) || (T2 == T1)) T2 = TimeCurrent() + 10 * PeriodSeconds();
      P1 = ObjectGetDouble(0, UpperBorder, OBJPROP_PRICE, 0);
      P2 = ObjectGetDouble(0, UpperBorder, OBJPROP_PRICE, 1);
      // Horizontal line case
      if (P2 == 0) P2 = P1;
      P3 = ObjectGetDouble(0, LowerBorder, OBJPROP_PRICE, 0);
      // Get object's height, making sure that the channel is "rayed" and switching ray properties back as they were.
      bool prev_ray_right_upper = ObjectGetInteger(0, UpperBorder, OBJPROP_RAY_RIGHT);
      bool prev_ray_right_lower = ObjectGetInteger(0, LowerBorder, OBJPROP_RAY_RIGHT);
      ObjectSetInteger(0, UpperBorder, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, LowerBorder, OBJPROP_RAY_RIGHT, true);
      double Top, Bottom;
      if (ObjectGetInteger(0, UpperBorder, OBJPROP_TYPE) == OBJ_HLINE) Top = P1;
      else Top = ObjectGetValueByTime(0, UpperBorder, T1, 0);
      if (ObjectGetInteger(0, LowerBorder, OBJPROP_TYPE) == OBJ_HLINE) Bottom = P3;
      else Bottom = ObjectGetValueByTime(0, LowerBorder, T1, 0);
      Height = Top - Bottom;
      ObjectSetInteger(0, UpperBorder, OBJPROP_RAY_RIGHT, prev_ray_right_upper);
      ObjectSetInteger(0, LowerBorder, OBJPROP_RAY_RIGHT, prev_ray_right_lower);
   }
   else
   {
      Print("Borders not found. Terminating script.");
      return;
   }
   
   Print("Channel height found: ", DoubleToString(MathAbs(Height), _Digits));

   double EntryDistance = Height * ((double)PercentageEntry / 100.0);
   if (Sides == Both)
   {
      if (ObjectFind(0, EntryName) < 0) ObjectCreate(0, EntryName, OBJ_CHANNEL, 0, T1, P1 + EntryDistance, T2, P2 + EntryDistance, T3, P3 - EntryDistance);
      else
      {
         ObjectSetInteger(0, EntryName, OBJPROP_TIME, 0, T1);
         ObjectSetDouble(0, EntryName, OBJPROP_PRICE, 0, P1 + EntryDistance);
         ObjectSetInteger(0, EntryName, OBJPROP_TIME, 1, T2);
         ObjectSetDouble(0, EntryName, OBJPROP_PRICE, 1, P2 + EntryDistance);
         ObjectSetInteger(0, EntryName, OBJPROP_TIME, 2, T3);
         ObjectSetDouble(0, EntryName, OBJPROP_PRICE, 2, P3 - EntryDistance);
      }
      ObjectSetInteger(0, EntryName, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, EntryName, OBJPROP_SELECTABLE, true);
      ObjectSetInteger(0, EntryName, OBJPROP_HIDDEN, false);
      ObjectSetInteger(0, EntryName, OBJPROP_COLOR, EntryColor);
      ObjectSetInteger(0, EntryName, OBJPROP_STYLE, EntryStyle);
   }
   else
   {
      double Price1 = 0, Price2 = 0;
      string Name;
      ENUM_OBJECT ObjType;
      if (Sides == Upper)
      {
         if (Height > 0)
         {
            Price1 = P1 + EntryDistance;
            Price2 = P2 + EntryDistance;
         }
         else
         {
            Price1 = P1 - Height - EntryDistance;
            Price2 = P2 - Height - EntryDistance;
         }
         Name = UpperEntry;
      }
      else if (Sides == Lower)
      {
         if (Height > 0)
         {
            Price1 = P1 - Height - EntryDistance;
            Price2 = P2 - Height - EntryDistance;
         }
         else
         {
            Price1 = P1 + EntryDistance;
            Price2 = P2 + EntryDistance;
         }
         Name = LowerEntry;
      }

      if (P1 == P2) ObjType = OBJ_HLINE;
      else ObjType = OBJ_TREND;
      if (ObjectFind(0, Name) == -1) ObjectCreate(0, Name, ObjType, 0, T1, Price1, T2, Price2);
      else
      {
         ObjectSetInteger(0, Name, OBJPROP_TIME, 0, T1);
         ObjectSetDouble(0, Name, OBJPROP_PRICE, 0, Price1);
         ObjectSetInteger(0, Name, OBJPROP_TIME, 1, T2);
         ObjectSetDouble(0, Name, OBJPROP_PRICE, 1, Price2);
      }
      ObjectSetInteger(0, Name, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, Name, OBJPROP_SELECTABLE, true);
      ObjectSetInteger(0, Name, OBJPROP_HIDDEN, false);
      ObjectSetInteger(0, Name, OBJPROP_COLOR, EntryColor);
      ObjectSetInteger(0, Name, OBJPROP_STYLE, EntryStyle);
   }
   
   Print("Entry positioned.");

   double TPDistance = Height * ((double)PercentageTP / 100.0);
   if (Sides == Both)
   {
      if (ObjectFind(0, TPName) < 0) ObjectCreate(0, TPName, OBJ_CHANNEL, 0, T1, P1 + TPDistance, T2, P2 + TPDistance, T3, P3 - TPDistance);
      else
      {
         ObjectSetInteger(0, TPName, OBJPROP_TIME, 0, T1);
         ObjectSetDouble(0, TPName, OBJPROP_PRICE, 0, P1 + TPDistance);
         ObjectSetInteger(0, TPName, OBJPROP_TIME, 1, T2);
         ObjectSetDouble(0, TPName, OBJPROP_PRICE, 1, P2 + TPDistance);
         ObjectSetInteger(0, TPName, OBJPROP_TIME, 2, T3);
         ObjectSetDouble(0, TPName, OBJPROP_PRICE, 2, P3 - TPDistance);
      }
      ObjectSetInteger(0, TPName, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, TPName, OBJPROP_SELECTABLE, true);
      ObjectSetInteger(0, TPName, OBJPROP_HIDDEN, false);
      ObjectSetInteger(0, TPName, OBJPROP_COLOR, TPColor);
      ObjectSetInteger(0, TPName, OBJPROP_STYLE, TPStyle);
   }
   else
   {
      double Price1 = 0, Price2 = 0;
      string Name;
      ENUM_OBJECT ObjType;
      if (Sides == Upper)
      {
         if (Height > 0)
         {
            Price1 = P1 + TPDistance;
            Price2 = P2 + TPDistance;
         }
         else
         {
            Price1 = P1 - Height - TPDistance;
            Price2 = P2 - Height - TPDistance;
         }
         Name = UpperTP;
      }
      else if (Sides == Lower)
      {
         if (Height > 0)
         {
            Price1 = P1 - Height - TPDistance;
            Price2 = P2 - Height - TPDistance;
         }
         else
         {
            Price1 = P1 + TPDistance;
            Price2 = P2 + TPDistance;
         }
         Name = LowerTP;
      }

      if (P1 == P2) ObjType = OBJ_HLINE;
      else ObjType = OBJ_TREND;
      if (ObjectFind(0, Name) == -1) ObjectCreate(0, Name, ObjType, 0, T1, Price1, T2, Price2);
      else
      {
         ObjectSetInteger(0, Name, OBJPROP_TIME, 0, T1);
         ObjectSetDouble(0, Name, OBJPROP_PRICE, 0, Price1);
         ObjectSetInteger(0, Name, OBJPROP_TIME, 1, T2);
         ObjectSetDouble(0, Name, OBJPROP_PRICE, 1, Price2);
      }
      ObjectSetInteger(0, Name, OBJPROP_RAY_RIGHT, true);
      ObjectSetInteger(0, Name, OBJPROP_SELECTABLE, true);
      ObjectSetInteger(0, Name, OBJPROP_HIDDEN, false);
      ObjectSetInteger(0, Name, OBJPROP_COLOR, TPColor);
      ObjectSetInteger(0, Name, OBJPROP_STYLE, TPStyle);
   }

   Print("TP positioned.");

   Print("Channel Pattern script finished.");
}
//+------------------------------------------------------------------+